// CkByteData.h: interface for the CkByteData class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _CKBYTEDATA_H
#define _CKBYTEDATA_H

#pragma once

#include "CkObject.h"

// The CkByteData class is a convenient class for holding a block of
// binary data, or non-null terminated text data.

// CLASS: CkByteData
class CkByteData : public CkObject
{
    public:
	CkByteData();
	~CkByteData();

	CkByteData(const CkByteData &);
	CkByteData &operator=(const CkByteData &);

	// BEGIN PUBLIC INTERFACE
	bool get_Utf8(void) const;
	void put_Utf8(bool b);


	// Clears the object of data.
	void clear(void);

	// Get the size in bytes.
        unsigned long getSize(void) const;

	// Get a pointer to the data.
        const unsigned char *getData(void) const;
        const unsigned char *getDataAt(int byteIndex) const;

	// Append more data
        void append(const unsigned char *data, unsigned long numBytes);
	void appendStr(const char *str);
	void appendChar(char ch);

	// Load a complete file into this object.  The contents of this object
	// are automatically cleared before the file is loaded, so the result
	// is a mirror image (in memory) of the bytes in the file.
        bool loadFile(const char *filename);
	bool saveFile(const char *filename);

	// Create a new file, or append to an existing file.
	bool appendFile(const char *filename);

	// Discards the last numBytes of data.
        void shorten(unsigned long numBytes);

	// The CkByteData will use *your* memory buffer, and will not delete
	// it when the object is destructed.  
	void borrowData(unsigned char *yourData, unsigned long numBytes);

	// Removes the data from the CkByteData object.  The caller will receive
	// a pointer to the memory buffer, and is responsible for deleting it.
	// (Example: 
	//	unsigned char *data = byteData.removeData();
	//	... do something with the data....
	//	delete [] data;
	unsigned char *removeData(void);

	// END PUBLIC INTERFACE

    private:
	// Internal implementation.
	void *m_impl;
	bool m_utf8;	// If true, all input "const char *" parameters are utf-8, otherwise they are ANSI strings.

    public:
	// Used internally by Chilkat.
	void *getImpl(void);
	const void *getImplC(void) const;

};


#endif
